import UIKit
import VIDVLiveness

@objc(Vidvliveness)
class Vidvliveness: NSObject {
    
    let dispatchGroup = DispatchGroup()
    var jsonResponse = ""
    var success = false
    var capturedActionsArray: [VIDVDetectedFace] = []

    @objc(startLiveness:withResolver:withRejecter:)
    func startLiveness(_ params: NSDictionary,_ resolve: @escaping RCTPromiseResolveBlock,_ reject: @escaping RCTPromiseRejectBlock) {
        
        let baseURL = params.value(forKey: "base_url") as? String ?? ""
        let accessToken = params.value(forKey: "access_token") as? String ?? ""
        let bundle = params.value(forKey: "bundle_key") as? String ?? ""
        
        let enableSmile  = params.value(forKey: "enable_smile") as? Bool ?? true
        let enableLeft  = params.value(forKey: "enable_look_left") as? Bool ?? true
        let enableRight  = params.value(forKey: "enable_look_right") as? Bool ?? true
        let enableEyes  = params.value(forKey: "enable_close_eyes") as? Bool ?? true
        
        let languageString = params.value(forKey: "language") as? String ?? ""
        let trials  = params.value(forKey: "livenss_number_of_failed_trials") as? Int
        let timer  = params.value(forKey: "liveness_time_per_action") as? Int
        let instructions  = params.value(forKey: "livness_number_of_instructions") as? Int
        
        let facematchImage  = params.value(forKey: "facematch_image") as? String
        let facematchOcrTransactionId  = params.value(forKey: "facematch_ocr_transactionId") as? String
        let enableVoiceover = params.value(forKey: "enable_voiceover") as? Bool ?? true
        
            // creds
        var settings = VIDVLivenessBuilder()
            .setBundleKey(bundle)
            .setAccessToken(accessToken)
            .setBaseURL(baseURL)
        
        // liveness actions
        if !enableEyes {
            settings = settings.withoutCloseEyes()
        }
        if !enableLeft {
            settings = settings.withoutLookLeft()
        }
        if !enableRight {
            settings = settings.withoutLookRight()
        }
        if !enableSmile {
            settings = settings.withoutSmile()
        }
        
        // liveness settings
        if languageString != "" {
            settings = settings.setLanguage(languageString)
        }
        if let trials = trials {
            settings = settings.setFailTrials(trials)
        }
        if let timer = timer {
            settings = settings.setInstructionTimer(timer)
        }
        if let instructionsCount = instructions {
            settings = settings.setNumberOfInstructions(instructionsCount)
        }
        if !enableVoiceover {
            settings = settings.withoutVoiceOver()
        }
        
        // face match
        if let facematchOcrTransactionId = facematchOcrTransactionId  {
            settings = settings.setFrontTransactionID(facematchOcrTransactionId)
        } else if let facematchImage = facematchImage {
            let data = Data(base64Encoded: facematchImage ) ?? Data()
            let image = UIImage(data: data)
            settings = settings.setFaceMatchImage(image)
        }
        
        DispatchQueue.main.async {
            if #available(iOS 11.0, *) {
                
                let keyWindow = UIApplication.shared.windows.filter {$0.isKeyWindow}.first
                guard var topController = keyWindow?.rootViewController else {return}
                while let presentedViewController = topController.presentedViewController {
                    topController = presentedViewController
                }
                settings.start(vc: topController, livenessDelegate: self)
            } else {
                // Fallback on earlier versions (we don't support earlier versions)
            }
        }
        self.dispatchGroup.enter()
        dispatchGroup.notify(queue: DispatchQueue.global()){
            if self.success {
                resolve(self.jsonResponse)
            } else {
                resolve(self.jsonResponse)
            }
        }
    }
    
}

extension IDVOCRReact : VIDVLivenessDelegate {    
    func capturedActions(_ action: VIDVLiveness.VIDVDetectedFace) {
        capturedActionsArray.append(action)
    }
    
    func livenessDidFinishProcess(_ VIDVLivenessResponse: VIDVLivenessResponse) {
        var dict = NSMutableDictionary()
        switch VIDVLivenessResponse {
        case .success(data: let data):
            let VIDVReslut = VIDVResultToDict(VIDVResult: data, hasResult: true, errorMessage: nil, errorCode: nil, stage: nil)
            dict["state"] = "SUCCESS"
            dict["errorCode"] = 0
            dict["errorMessage"] = ""
            dict["valifyData"] = VIDVReslut
            success = true
            returnData(dict: dict)
        case .serviceFailure(data: let data, errorCode: let errorCode, errorMessage: let errorMessage):
            let VIDVReslut = VIDVResultToDict(VIDVResult: data, hasResult: true, errorMessage: errorMessage, errorCode: errorCode, stage: nil)
            dict["state"] = "FAILURE"
            dict["errorCode"] = errorCode
            dict["errorMessage"] = ""
            dict["valifyData"] = VIDVReslut
            success = false
            returnData(dict: dict)
        case .builderError(errorCode: let errorCode, errorMessage: let errorMessage):
            let VIDVReslut = VIDVResultToDict(VIDVResult: nil, hasResult: false, errorMessage: errorMessage, errorCode: errorCode, stage: nil)
            dict["state"] = "ERROR"
            dict["errorCode"] = errorCode
            dict["errorMessage"] = ""
            dict["valifyData"] = VIDVReslut
            success = false
            returnData(dict: dict)
        case .userExited(data: let data, step: let step):
            let VIDVReslut =  VIDVResultToDict(VIDVResult: data, hasResult: true, errorMessage: nil, errorCode: 0, stage: step)
            dict["state"] = "EXIT"
            dict["errorCode"] = 0
            dict["errorMessage"] = ""
            dict["valifyData"] = VIDVReslut
            success = false
            returnData(dict: dict)
        @unknown default:
            break;
        }
    }
    
    func VIDVResultToDict(VIDVResult: VIDVLivenessData?, hasResult: Bool, errorMessage: String?, errorCode: Int?, stage: String?) -> NSMutableDictionary {
        
        let dict = NSMutableDictionary()
        let nameValuePairs = NSMutableDictionary()
        
        dict["nameValuePairs"] = nameValuePairs
        
        
        if errorCode != nil {
            nameValuePairs["errorCode"] = errorCode
        }
        if errorMessage != nil {
            nameValuePairs["errorMessage"] = errorMessage
        }
        if stage != nil {
            nameValuePairs["step"] = stage
        }
        dict["captured_images"] = self.capturedActionsArray
        
        if hasResult {
            do {
                let jsonEncoder = JSONEncoder()
                let jsonData = try jsonEncoder.encode(VIDVResult)
                guard let dictionary = try JSONSerialization.jsonObject(with: jsonData, options: .allowFragments) as? [String: Any] else {
                    throw NSError()
                }
                nameValuePairs["valifyData"] = dictionary
            } catch {
                print("Failed to encode VIDV Result: \(error.localizedDescription)")
            }
        }
        
        return dict
    }
    
    func returnData(dict: NSMutableDictionary){
        do {
            let jsonData = try JSONSerialization.data(withJSONObject: dict, options: .prettyPrinted)
            let json = String(data: jsonData, encoding: .utf8)
            self.jsonResponse = json ?? ""
            dispatchGroup.leave()
        } catch let error as NSError {
            print("Failed to load: \(error.localizedDescription)")
        }
    }
}